local name, private = ...

local list = private.objects.list
local queue = private.objects.queue
local dict = private.objects.dict

local _, _, _, TOC_VERSION = GetBuildInfo()
private.TOC_VERSION = TOC_VERSION

private.baseArtifacts = {
    -- NOTE: These are ITEM IDs, not artifact info.
    { -- 1 (Dwarf)
        common = {
            63113, -- Belt Buckle with Anvilmar Crest
            64339, -- Bodacious Door Knocker
            63112, -- Bone Gaming Dice
            64340, -- Boot Heel with Scrollwork
            63409, -- Ceramic Funeral Urn
            64362, -- Dented Shield of Horuz Killcrow
            66054, -- Dwarven Baby Socks
            64342, -- Golden Chamber Pot
            64344, -- Ironstar's Petrified Shield
            64368, -- Mithril Chain of Angerforge
            63414, -- Moltenfist's Jeweled Goblet
            64337, -- Notched Sword of Tunadil the Redeemer
            63408, -- Pewter Drinking Cup
            64659, -- Pipe of Franclorn Forgewright
            64487, -- Scepter of Bronzebeard
            64367, -- Scepter of Charlga Razorflank
            64366, -- Scorched Staff of Shadow Priest Anund
            64483, -- Silver Kris of Korl
            63411, -- Silver Neck Torc
            64371, -- Skull Staff of Shadowforge
            64485, -- Spiked Gauntlets of Anvilrage
            63410, -- Stone Gryphon
            64484, -- Warmaul of Burningeye
            64343, -- Winged Helm of Corehammer
            63111, -- Wooden Whistle
            64486, -- Word of Empress Zoe
            63110, -- Worn Hunting Knife
        },
        rare = {
            64373, -- Chalice of the Mountain Kings
            64372, -- Clockwork Gnome
            64489, -- Staff of Sorcerer-Thane Thaurissan
            64488, -- The Innkeeper's Daughter
        },
    },
    { -- 2 (Draenei)
        common = {
            64440, -- Anklet with Golden Bells
            64453, -- Baroque Sword Scabbard
            64442, -- Carved Harp of Exotic Wood
            64455, -- Dignified Portrait
            64454, -- Fine Crystal Candelabra
            64458, -- Plated Elekk Goad
            64444, -- Scepter of the Nathrezim
            64443, -- Strange Silver Paperweight
        },
        rare = {
            64456, -- Arrival of the Naaru
            64457, -- The Last Relic of Argus
        },
    },
    { -- 3 (Fossil)
        common = {
            64355, -- Ancient Shark Jaws
            63121, -- Beautiful Preserved Fern
            63109, -- Black Trilobite
            64349, -- Devilsaur Tooth
            64385, -- Feathered Raptor Arm
            64473, -- Imprint of a Kraken Tentacle
            64350, -- Insect in Amber
            64468, -- Proto-Drake Skeleton
            66056, -- Shard of Petrified Wood
            66057, -- Strange Velvet Worm
            63527, -- Twisted Ammonite Shell
            64387, -- Vicious Ancient Fish
        },
        rare = {
            60955, -- Fossilized Hatchling
            60954, -- Fossilized Raptor
        },
    },
    { -- 4 (Night Elf)
        common = {
            64647, -- Carcanet of the Hundred Magi
            64379, -- Chest of Tiny Glass Animals
            63407, -- Cloak Clasp with Antlers
            63525, -- Coin from Eldre'Thalas
            64381, -- Cracked Crystal Vial
            64357, -- Delicate Music Box
            63528, -- Green Dragon Ring
            64356, -- Hairpin of Silver and Malachite
            63129, -- Highborne Pyxis
            63130, -- Inlaid Ivory Comb
            64354, -- Kaldorei Amphora
            66055, -- Necklace with Elune Pendant
            63131, -- Scandalous Silk Nightgown
            64382, -- Scepter of Xavius
            63526, -- Shattered Glaive
            64648, -- Silver Scroll Case
            64378, -- String of Small Pink Pearls
            64650, -- Umbra Crescent
        },
        rare = {
            64646, -- Bones of Transformation
            64361, -- Druid and Priest Statue Set
            64358, -- Highborne Soul Mirror
            64383, -- Kaldorei Wind Chimes
            64643, -- Queen Azshara's Dressing Gown
            64645, -- Tyrande's Favorite Doll
            64651, -- Wisp Amulet
        },
    },
    { -- 5 (Nerubian)
        common = {
            64479, -- Ewer of Jormungar Blood
            64477, -- Gruesome Heart Box
            64476, -- Infested Ruby Ring
            64475, -- Scepter of Nezar'Azret
            64478, -- Six-Clawed Cornice
            64474, -- Spidery Sundial
            64480, -- Vizier's Scrawled Streamer
        },
        rare = {
            64481, -- Blessing of the Old God
            64482, -- Puzzle Box of Yogg-Saron
        },
    },
    { -- 6 (Orc)
        common = {
            64436, -- Fiendish Whip
            64421, -- Fierce Wolf Figurine
            64418, -- Gray Candle Stub
            64417, -- Maul of Stone Guard Mur'og
            64419, -- Rusted Steak Knife
            64420, -- Scepter of Nekros Skullcrusher
            64438, -- Skull Drinking Cup
            64437, -- Tile of Glazed Clay
            64389, -- Tiny Bronze Scorpion
        },
        rare = {
            64644, -- Headdress of the First Shaman
        },
    },
    { -- 7 (Tol["vir)
        common = {
            64657, -- Canopic Jar
            64652, -- Castle of Sand
            64653, -- Cat Statue with Emerald Eyes
            64656, -- Engraved Scimitar Hilt
            64658, -- Sketch of a Desert Palace
            64654, -- Soapstone Scarab Necklace
            64655, -- Tiny Oasis Mosaic
        },
        rare = {
            60847, -- Crawling Claw
            64881, -- Pendant of the Scarab Storm
            64904, -- Ring of the Boy Emperor
            64883, -- Scepter of Azj'Aqir
            64885, -- Scimitar of the Sirocco
            64880, -- Staff of Ammunae
        },
    },
    { -- 8 (Troll)
        common = {
            64348, -- Atal'ai Scepter
            64346, -- Bracelet of Jade and Coins
            63524, -- Cinnabar Bijou
            64375, -- Drakkari Sacrificial Knife
            63523, -- Eerie Smolderthorn Idol
            63413, -- Feathered Gold Earring
            63120, -- Fetish of Hir'eek
            66058, -- Fine Bloodscalp Dinnerware
            64347, -- Gahz'rilla Figurine
            63412, -- Jade Asp with Ruby Eyes
            63118, -- Lizard Foot Charm
            64345, -- Skull-Shaped Planter
            64374, -- Tooth with Gold Filling
            63115, -- Zandalari Voodoo Doll
        },
        rare = {
            64377, -- Zin'rokh, Destroyer of Worlds
        },
    },
    { -- 9 (Vrykul)
        common = {
            64464, -- Fanged Cloak Pin
            64462, -- Flint Striker
            64459, -- Intricate Treasure Chest Key
            64461, -- Scramseax
            64467, -- Thorned Necklace
        },
        rare = {
            64460, -- Nifflevar Bearded Axe
        },
    },
}

private.patchedArtifacts = {
    [40100] = {
        { common = {}, rare = {}, }, -- 1 (Dwarf)
        { common = {}, rare = {}, }, -- 2 (Draenei)
        { -- 3 (Fossil)
            common = {},
            rare = {
                69821, -- Pterrordax Hatchling
                69776, -- Ancient Amber
                69764, -- Extinct Turtle Shell
            },
        },
        { common = {}, rare = {}, }, -- 4 (Night Elf)
        { common = {}, rare = {}, }, -- 5 (Nerubian)
        { common = {}, rare = {}, }, -- 6 (Orc)
        { common = {}, rare = {}, }, -- 7 (Tol'vir)
        { -- 8 (Troll)
            common = {},
            rare = {
                69824, -- Voodoo Figurine
                69777, -- Haunted War Drum
            },
        },
        { -- 9 (Vrykul)
            common = {},
            rare = {
                69775, -- Vrykul Drinking Horn
            },
        },
    },
}

-- Use the list of base and patched artifacts to build a complete list of artifacts
private.artifacts = {}
private.artifacts.isProcessed = false
function private.artifacts:process()
    if not private.artifacts.isProcessed then
        for i = 1, #private.baseArtifacts do
            -- First, copy each race from the base list into the full table
            private.artifacts[i] = {}
            private.artifacts[i].common = list()
            for k, v in pairs(private.baseArtifacts[i].common) do
                private.artifacts[i].common:add(v)
                -- Cache the item data
                GetItemInfo(v)
            end
            private.artifacts[i].rare = list()
            for k, v in pairs(private.baseArtifacts[i].rare) do
                private.artifacts[i].rare:add(v)
                -- Cache the item data
                GetItemInfo(v)
            end
        end
        
        for k, v in pairs(private.patchedArtifacts) do
            -- Then, copy every artifact added in a patch that is currently active into this list
            if k <= TOC_VERSION then
                for i = 1, #v do
                    if i > #private.artifacts then
                        private.artifacts[i] = {}
                        private.artifacts[i].common = list()
                        private.artifacts[i].rare = list()
                    end
                    for kk, vv in pairs(v[i].common) do
                        private.artifacts[i].common:add(vv)
                        -- Cache the item data
                        GetItemInfo(vv)
                    end
                    for kk, vv in pairs(v[i].rare) do
                        private.artifacts[i].rare:add(vv)
                        -- Cache the item data
                        GetItemInfo(vv)
                    end
                end
            end
        end
        
        private.artifacts.isProcessed = true
    end
end

private.baseLinks = {
    -- ["Item name"] = "Project name"
    -- English
    ["Crawling Claw"] = "Mummified Monkey Paw",
    -- French
    ["Epingle à cheveux en argent et malachite"] = "Épingle à cheveux en argent et malachite",
    ["Etrange presse-papier en argent"] = "Étrange presse-papier en argent",
    ["Couteau de chasse usé"] = "Couteau de chasseur usé",
    ["Dé de jeu en os"] = "Dé en os",
    ["Collier avec pendentif d’Élune"] = "Collier avec pendentif d'Elune", -- This one was troublesome - it's the apostrophe...
}

private.travel = {
    spells = {
        -- spell ID, cooldown (in seconds), reagent (0 if none)
        { 556, 900, 0 }, -- Astral Recall
        { 3561, 0, 17031 }, -- Teleport: Stormwind
        { 3562, 0, 17031 }, -- Teleport: Ironforge
        { 3565, 0, 17031 }, -- Teleport: Darnassus
        { 32271, 0, 17031 }, -- Teleport: Exodar
        { 49359, 0, 17031 }, -- Teleport: Theramore
        { 3567, 0, 17031 }, -- Teleport: Orgrimmar
        { 3563, 0, 17031 }, -- Teleport: Undercity
        { 3566, 0, 17031 }, -- Teleport: Thunder Bluff
        { 32272, 0, 17031 }, -- Teleport: Silvermoon
        { 49358, 0, 17031 }, -- Teleport: Stonard
        { 33690, 0, 17031 }, -- Teleport: Shattrath (Alliance)
        { 35715, 0, 17031 }, -- Teleport: Shattrath (Horde)
        { 53140, 0, 17031 }, -- Teleport: Dalaran
        { 88342, 0, 17031 }, -- Teleport: Tol Barad (Alliance)
        { 88344, 0, 17031 }, -- Teleport: Tol Barad (Horde)
        { 18960, 0, 0 }, -- Teleport: Moonglade
        { 50977, 60, 0 }, -- Death Gate
    },
    items = {
        -- item ID, spell ID, cooldown (in seconds), consumed (boolean)
        { 6948, 8690, 1800, false }, -- Hearthstone
        { 28585, 39937, 1800, false }, -- Ruby Slippers
        { 54452, 75136, 1800, false }, -- Ethereal Portal
        { 64488, 94719, 1800, false }, -- The Innkeeper's Daughter
        { 37118, 48129, 1200, true }, -- Scroll of Recall
        { 44314, 60320, 1200, true }, -- Scroll of Recall II
        { 44315, 60321, 1200, true }, -- Scroll of Recall III
        { 58487, 80256, 60, true }, -- Potion of Deepholm
        { 18986, 23453, 14400, false }, -- Ultrasafe Transporter - Gadgetzan
        { 30544, 36941, 14400, false }, -- Ultrasafe Transporter - Toshley's Station
        { 18984, 23442, 14400, false }, -- Dimensional Ripper - Everlook
        { 30542, 36890, 14400, false }, -- Dimensional Ripper - Area 52
        { 48933, 67833, 14400, false }, -- Wormhole Generator: Northrend
        { 50287, 71436, 86400, false }, -- Boots of the Bay
        { 63352, 89157, 28800, false }, -- Shroud of Cooperation (Stormwind)
        { 63353, 89158, 28800, false }, -- Shroud of Cooperation (Orgrimmar)
        { 63206, 89157, 14400, false }, -- Wrap of Unity (Stormwind)
        { 63207, 89158, 14400, false }, -- Wrap of Unity (Orgrimmar)
        { 65360, 89157, 7200, false }, -- Cloak of Coordination (Stormwind)
        { 65274, 89158, 7200, false }, -- Cloak of Coordination (Orgrimmar)
        { 52251, 73324, 3600, false }, -- Jaina's Locket
        { 40586, 54406, 1800, false }, -- Band of the Kirin Tor
        { 45688, 54406, 1800, false }, -- Inscribed Band of the Kirin Tor
        { 48954, 54406, 1800, false }, -- Etched Band of the Kirin Tor
        { 51560, 54406, 1800, false }, -- Runed Band of the Kirin Tor
        { 44935, 54406, 1800, false }, -- Ring of the Kirin Tor
        { 45690, 54406, 1800, false }, -- Inscribed Ring of the Kirin Tor
        { 48956, 54406, 1800, false }, -- Etched Ring of the Kirin Tor
        { 51559, 54406, 1800, false }, -- Runed Ring of the Kirin Tor
        { 44934, 54406, 1800, false }, -- Loop of the Kirin Tor
        { 45689, 54406, 1800, false }, -- Inscribed Loop of the Kirin Tor
        { 48955, 54406, 1800, false }, -- Etched Loop of the Kirin Tor
        { 51558, 54406, 1800, false }, -- Runed Loop of the Kirin Tor
        { 40585, 54406, 1800, false }, -- Signet of the Kirin Tor
        { 45691, 54406, 1800, false }, -- Inscribed Signed of the Kirin Tor
        { 48957, 54406, 1800, false }, -- Etched Signet of the Kirin Tor
        { 51557, 54406, 1800, false }, -- Runed Signet of the Kirin Tor
    },
}

private.onUseItems = {
    -- item ID, spell ID, cooldown (in seconds), consumed (boolean)
    -- Dwarf
    { 64373, 94690, 600, false }, -- Chalice of the Mountain Kings
    { 64488, 94719, 1800, false }, -- The Innkeeper's Daughter
    -- Draenei
    { 64457, 82674, 43200, false }, -- The Last Relic of Argus
    { 64456, 91752, 180, false }, -- Arrival of the Naaru
    -- Fossil
    { 69776, 98445, 1800, false }, -- Ancient Amber
    -- Night Elf
    { 64358, 95007, 600, false }, -- Highborne Soul Mirror
    { 64383, 91749, 60, false }, -- Kaldorei Wind Chimes
    { 64361, 91743, 900, false }, -- Druid and Priest Statue Set
    { 64646, 91760, 600, false }, -- Bones of Transformation
    { 64651, 91771, 600, false }, -- Wisp Amulet
    -- Nerubian
    { 64481, 91754, 600, false }, -- Blessing of the Old God
    { 64482, 91756, 60, false }, -- Puzzle Box of Yogg-Saron
    -- Orc
    -- Nothing for orc...
    -- Tol'vir
    { 64881, 92146, 600, false }, -- Pendant of the Scarab Storm
    -- Troll
    { 69777, 98447, 1800, false }, -- Haunted War Drum
    -- Vrykul
    { 69775, 98444, 0, false }, -- Vrykul Drinking Horn
    -- Learnable items (pets, mounts)
    { 64372, 90523, 0, true }, -- Clockwork Gnome
    { 69821, 98571, 0, true }, -- Pterrordax Hatchling
    { 60955, 84752, 0, true }, -- Fossilized Hatchling
    { 60954, 84751, 0, true }, -- Fossilized Raptor
    { 60847, 84263, 0, true }, -- Crawling Claw
    { 64883, 92155, 0, true }, -- Scepter of Azj'Aqir
    { 69824, 98587, 0, true }, -- Voodoo Figurine
}

-- Cache the travel spells/items
for i=1, #private.travel.items do
    GetItemIcon(private.travel.items[i][1])
    GetItemInfo(private.travel.items[i][1])
end
for i=1, #private.travel.spells do
    GetSpellInfo(private.travel.spells[i][1])
end

-- Currencies
private.currencies = {
    [384] = 1, -- Dwarf Archaeology Fragment
    [398] = 2, -- Draenei Archaeology Fragment
    [393] = 3, -- Fossil Archaeology Fragment
    [394] = 4, -- Night Elf Archaeology Fragment
    [400] = 5, -- Nerubian Archaeology Fragment
    [397] = 6, -- Orc Archaeology Fragment
    [401] = 7, -- Tol'vir Archaeology Fragment
    [385] = 8, -- Troll Archaeology Fragment
    [399] = 9, -- Vrykul Archaeology Fragment
}

private.continents = {
    { 1, 3, 4, 7, 8 }, -- Kalimdor
    { 1, 3, 4, 5, 8 }, -- Eastern Kingdoms
    { 2, 6 }, -- Outland
    { 4, 5, 8, 9 }, -- Northrend
}
private.continents[0] = { 1, 2, 3, 4, 5, 6, 7, 8, 9, } -- Failsafe

-- Achievement order
private.achievementOrder = list({
    { -- The skill achievements
        "skill journeyman",
        "skill expert",
        "skill artisan",
        "skill master",
        "skill grand master",
        "skill illustrious",
    },
    { -- The rare counts
        "rare 1",
        "rare 10",
        "rare 20",
    },
    { -- The common counts
        "common 10",
        "common 50",
        "common 100",
    },
    "race count",
    "seven scepters",
    "kings under the mountain",
    "tragedy in three acts",
    "the harder they fall",
    "blue streak",
    "the boy who would be king",
})

-- In later patches/expansions, use add() to add further achievement categories